C-----------------------------------------------------------------------
C     Copyright (C) 2007, ENPC - INRIA - EDF R&D
C     Author(s): Maryline Tombette
C     
C     This file is part of the Simple Aqueous model (SIMPLE_AQUEOUS), a
C     component of the air quality modeling system Polyphemus.
C    
C     Polyphemus is developed in the INRIA - ENPC joint project-team
C     CLIME and in the ENPC - EDF R&D joint laboratory CEREA.
C    
C     Polyphemus is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published
C     by the Free Software Foundation; either version 2 of the License,
C     or (at your option) any later version.
C     
C     Polyphemus is distributed in the hope that it will be useful, but
C     WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
C     General Public License for more details.
C     
C     For more information, visit the Polyphemus web site:
C     http://cerea.enpc.fr/polyphemus/
C-----------------------------------------------------------------------

      subroutine simple_aqueous_module(ns,ns_aer,nbin_aer,RHO_AERO,
     &     fixed_rho_aero,DBF_AERO,dsf_aero,xbf_aero,ZA,HUMID,
     &     press_in_pa,temp,lwc_c,t0,t1,rain_rate,pH,qscav)

C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This routine solves the aqueous-phase model.
C     
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     HUMID : specific humidity    ([kg.kg^{-1}]).
C     PRESS_IN_ATM: pressure             ([Pa]).
C     TEMP  : temperate            ([T]).
C     LWC_C : liquid water content ([g.m^{-3}]).
C     T0/T1 : initial/final time   ([s]).
C     rain_rate : rain precipitation rate ([mm/hr]).
C     
C     
C     -- INPUT/OUTPUT VARIABLES
C     
C     ZA: gas-phase and aerosol concentration ([\mu.g/m^3]).     
C     
C     -- OUTPUT VARIABLES
C     
C     PH : pH.
C     qscav : scavenged quantity by in-cloud scavenging
C     for dissolved aerosols species ([\mu.g/m^3])
C     and dissolves gas      species ([ppm])
C     
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     Marilyne Tombette, CEREA, , on the basis of Pandis/Seinfeld book, 
C     pp. 
C     
C------------------------------------------------------------------------

      IMPLICIT NONE

      include 'data_mass.inc'
      include 'pointer_ctm.inc'
      include 'aerpar_simple.inc'
      include 'droppar_simple.inc'
      include 'pointer.inc'
      include 'num_aq.inc'
      include 'CONST_A.INC'

      INTEGER ns,ns_aer,nbin_aer

      double precision coefloc,tinit,coefloc2, coef_cte
      double precision dh2SO4,dso2
      double precision gas(ngas_aq), aerosol(nbin_aer,naers)
      double precision dsulfate
      double precision henrys(ngas_aq)
      double precision cst_dissoc(nreact_dissoc)
      double precision cst_oxydation(nreact_oxydation)
      double precision gasav(ngas_aq), aerosav(nbin_aer,naers)   
      double precision rh,temp,press_in_atm,lwc_c,lwc,t0,t1,initso2
      double precision dnew(nbin_aer),deltat,press_in_pa
      double precision numberconc(nbin_aer),totmass(nbin_aer)
      double precision ZA(ns+nbin_aer*ns_aer)
      double precision HUMID
      double precision tnitold, ammonold,sulfateold
      double precision nitbef,nitaf
      double precision sulfbef,sulfaf,sbal
      double precision pH
      double precision qscav(ns+nbin_aer*ns_aer),rain_rate
      double precision fdist(nbin_aer), fdist2(nbin_aer)
      double precision conc_in(5), conc_out(10)

      double precision collision_eff,drop_diam,scav_coef
      double precision so2g,henrys_eff_so2
      double precision scav_frac,aqSO2,aqH2O2,aqO3
      double precision DBF_AERO(nbin_aer+1),RHO_AERO(nbin_aer)
      double precision dsf_aero(nbin_aer),xbf_aero(nbin_aer+1)
      double precision fixed_rho_aero,totmasstot

      double precision dsivdt_o3,dsivdt_h2o2,dsivdt
      double precision tot_aer(naers),tot_sulf,tot_sulf2,aa
      double precision tot_nit,tot_amm,report,fo3,fh2o2
      double precision ptotso2
      
      integer ifirstact
      integer isect,isp,i,istep,ii,nistep,ind_ok,j,iaers
      integer icount,nact

      data collision_eff /0.9d0/

      double precision cst_pr,cst_pr2,cst_RT
      integer nesp

c     Initialisation
c     --------------

      cst_pr = 1.987d-3         ! perfect gas constant in kcal.K-1.mol-1
      cst_pr2 = 8.206d-2        ! perfect gas constant in atm.L.K-1.mol-1
      cst_RT = cst_pr2 * temp
      
      nesp = ns+nbin_aer*ns_aer

      press_in_atm  = press_in_pa/101325.d0 ! pressure in atm
      
      lwc = lwc_c * 1.d-6       ! in L_water/L_air

      call initactiv(nbin_aer,dsf_aero,ifirstact)

      nact = nbin_aer-ifirstact+1

      nistep = NITSUBAQ

      tot_sulf=0.d0

      do i=1,ngas_aq
         gas(i)=0.d0
      enddo
      
      dsulfate=0.d0
      do isect=1,nbin_aer
         totmass(isect)=0.d0
      enddo

      do isp=1,NESP
         qscav(isp)=0.d0
      enddo

      deltat = (t1-t0)/60.d0/nistep !timestep in min

      do isect=1,nbin_aer
         do isp=1,naers
            aerosol(isect,isp)=0.d0
         enddo
      enddo

C     Compute henry's and equilibrium constants.
C     ------------------------------------------
      
      coef_cte= (1.d0/temp - 1.d0/temp_ref)
      do i=1,ngas_aq
         henrys(i) = chenry(i) * exp(-dhhenry(i)*coef_cte / cst_pr)
      enddo
      do i=1,nreact_dissoc
         cst_dissoc(i) = ckdissoc(i) 
     $        * exp(dhkdissoc(i)*coef_cte)
      enddo
      do i=1,nreact_oxydation
         cst_oxydation(i) = ckoxydation(i) 
     $        * exp(-dhkoxydation(i)*coef_cte 
     $        / cst_pr)
      enddo
      
C     Compute sectional diameters and bimodal distribution
C     -------------------------------------------------------

C     convert \mu g.m-3 -> atm
      gas(igso2)  = ZA(ictmSO2)  *1.d-9*cst_RT/mmSO2  !SO2
      gas(ignh3)  = ZA(ictmNH3)  *1.d-9*cst_RT/mmNH3  !NH3
      gas(ighno3) = ZA(ictmHNO3) *1.d-9*cst_RT/mmHNO3 !HNO3
      gas(igh2o2) = ZA(ictmH2O2) *1.d-9*cst_RT/mmH2O2 !H2O2
      gas(igo3)   = ZA(ictmO3)   *1.d-9*cst_RT/mmO3   !O3 
      gas(igCO2) = 350.d0 * 1.d-6 * press_in_atm ! 350 ppm in averaged.

C     totmass en \mu g.m-3 without sulfate
      DO i = 1,nbin_aer
         ii = ns + i
         aerosol(i,nas) = ZA(ii+(ENa-1)*nbin_aer) ! Na
         aerosol(i,na4) = ZA(ii+(ESO4-1)*nbin_aer) ! SO4
         aerosol(i,naa) = ZA(ii+(ENH3-1)*nbin_aer) ! NH4
         aerosol(i,nan) = ZA(ii+(ENO3-1)*nbin_aer) ! NO3
         aerosol(i,nac) = ZA(ii+(ECl-1)*nbin_aer) ! Cl    
         aerosol(i,nar) = ZA(ii+(EMD-1)*nbin_aer) ! DUST
         aerosol(i,nae) = ZA(ii+(EBC-1)*nbin_aer) ! EC
         aerosol(i,nao) = ZA(ii+(EPOA-1)*nbin_aer) ! POA
         aerosol(i,naw) = ZA(ii+(EH2O-1)*nbin_aer) ! H2O 
      ENDDO
      
C     Transfer all H2SO4(g) to aerosol phase      
C     ------------------------------------------
      dh2SO4 = (ZA(ictmH2SO4)/nbin_aer) * mmSO4/mmH2SO4
      DO i=1,nbin_aer
         aerosol(i,na4) = aerosol(i,na4)+dh2SO4
      ENDDO
      ZA(ictmH2SO4) = 0.d0

      DO i = 1,nbin_aer
         totmass(i)=aerosol(i,naa)+aerosol(i,na4)+
     &        aerosol(i,nan)+aerosol(i,nac)+aerosol(i,nas)+
     &        aerosol(i,nao)+aerosol(i,nae)+aerosol(i,nar)
         numberconc(i)=totmass(i)/(dsf_aero(i)**3.d0)
     &        /cst_pi6/RHO_AERO(i)
      ENDDO

C     Calculation of the bulk distribution factors fdist
      totmasstot = 0.D0
      do i=ifirstact,nbin_aer
         totmasstot = totmass(i) + totmasstot
      enddo
      do i=1,ifirstact-1
         fdist(i) = 0.d0
      enddo
      do i=ifirstact,nbin_aer
         fdist(i) = totmass(i)/totmasstot
      enddo
     
C     Compute total sulfate, ammonium and nitrate.
C     -------------------------------------------
      tot_sulf = 0.d0
      tot_nit = 0.d0
      tot_amm = 0.d0
      DO i=ifirstact,nbin_aer
         tot_sulf = tot_sulf + aerosol(i,na4) 
         tot_nit = tot_nit + aerosol(i,nan) 
         tot_amm = tot_amm + aerosol(i,naa) 
      enddo
      
      sulfateold=tot_sulf
      tnitold=tot_nit
      ammonold=tot_amm

C     Compute relative humidity
C     ----------------------------

      call COMPUTE_RELATIVE_HUMIDITY(HUMID,temp,press_in_pa,rh)
      rh = DMIN1(DMAX1(rh, 0.01d0), 0.97d0)


C     ***************************************************
C     Begin computations
C     ***************************************************
      
      ind_ok = 0


c     Simulate cloud/fog
c     ----------------------

      conc_in(1)=1.d-10              ! S(IV) in mol.L-1_water
      conc_in(2) = tot_sulf*1.d-9/mmSO4/lwc ! S(VI) in mol.L-1_water
      conc_in(3) = tot_amm*1.d-9*cst_RT/mmNH4 + gas(ignh3) ! in atm
      conc_in(4) = tot_nit*1.d-9*cst_RT/mmNO3 + gas(ighno3) ! in atm
      conc_in(5) = gas(igco2)

C     First computation of pH
C     -----------------------
      call compute_ph(conc_in,henrys,cst_dissoc,temp,
     $     lwc,ph,conc_out)

C     Loop on time step for aqueous module.
C     -------------------------------------

      do istep = 1,nistep

         tinit= (istep-1)*deltat
C     Compute SO2 dissociation
C     Assume it is instantaneous
C     --------------------------
         henrys_eff_so2 = henrys(igso2) * (1
     $        + cst_dissoc(2)/conc_out(1)
     $        + cst_dissoc(2) * cst_dissoc(3)
     $        /(conc_out(1)*conc_out(1)))
         ptotso2 = gas(igso2) + lwc * cst_RT * conc_in(1)
         so2g =  ptotso2  / (1 + cst_RT 
     $        * henrys_eff_so2 * lwc)
         aqso2 = henrys(igso2) * so2g
         conc_in(1) = (ptotso2 - so2g)/cst_RT/lwc
         gas(igso2) = so2g
         call compute_ph(conc_in,henrys,cst_dissoc,
     $        temp,lwc,ph,conc_out)
         aqh2o2 = gas(igh2o2) * henrys(igh2o2)
         aqo3 = gas(igo3) * henrys(igo3)

C     Compute reaction rates of S(IV) with O3 and H2O2
C     ------------------------------------------------

C     rate Seinfel Pandis for O3

         dsivdt_o3 = - (cst_oxydation(1)*conc_out(2) !SO2 aq
     $        + cst_oxydation(2)*conc_out(3) !HSO3-
     $        + cst_oxydation(3)*conc_out(4)) !SO3--
     $        * aqo3 

C     rate VSRM for H2O2

         dsivdt_h2o2 = - 1300000.d0* exp( -4430.d0* coef_cte)
     $        * conc_out(2) * aqh2o2
     $        / (1.d0+16.d0*conc_out(1))

C     total rate

         dsivdt = dsivdt_o3 + dsivdt_h2o2

         if (-dsivdt*deltat.gt.conc_in(1)) then
            fo3 = dsivdt_o3/dsivdt
            fh2o2 = dsivdt_h2o2/dsivdt
            dsivdt = - conc_in(1)/deltat
            dsivdt_o3 = fo3 * dsivdt
            dsivdt_h2o2 = fh2o2 * dsivdt
            conc_in(1) = 0.d0
         else
            conc_in(1) = conc_in(1) + dsivdt*deltat
         endif

C     Update concentrations
C     ---------------------
         conc_in(2) = conc_in(2) - dsivdt*deltat 
         gas(igo3) = gas(igo3)+dsivdt_o3*lwc *deltat
         gas(igh2o2) = gas(igh2o2)+dsivdt_h2o2*lwc *deltat 
         gas(ighno3) = conc_out(9)
         gas(ignh3) = conc_out(10)

      enddo

C     New total sulfate, nitrate and ammonium.
C     ----------------------------------------

      tot_sulf = conc_in(2) * 1.d9 * mmSO4 * lwc
      tot_nit= conc_out(6) * mmNO3 * 1.d9 *lwc 
      tot_amm= conc_out(8) * mmNH4 * 1.d9 *lwc 
      
      gas(ignh3) = conc_in(3) - tot_amm*1.d-9*cst_RT/mmNH4
      gas(igso2)= gas(igso2) + conc_in(1)*lwc*mmSO2/mmSO4

C     Compute new distribution by projection to
C     the aerosol population (as in VSRM).
C     ----------------------------------------------
     
      do isect=1,nbin_aer
         aerosol(isect,nan)    = aerosol(isect,nan)
     &        + fdist(isect) * (tot_nit - tnitold) ! NITRATE (aq) 
         aerosol(isect,naa)    = aerosol(isect,naa) 
     &        + fdist(isect) * (tot_amm - ammonold) ! AMMONIUM (aq)
         aerosol(isect,na4)    = aerosol(isect,na4)
     &        + fdist(isect) * (tot_sulf - sulfateold) ! SULFATE (aq)
      enddo

      do isp=1, naers
         do isect=1,nbin_aer
            aerosol(isect,isp)=dmax1(aerosol(isect,isp),tinyaq2)
         enddo
      enddo     

      do i = 1,nbin_aer
         totmass(i)=aerosol(i,naa)+aerosol(i,na4)+
     &        aerosol(i,nan)+aerosol(i,nac)+aerosol(i,nas)+
     &        aerosol(i,nao)+aerosol(i,nae)+aerosol(i,nar)
         dnew(i)=(totmass(i)/numberconc(i)
     &        /rho_aero(i)/cst_pi6)**cst_FRAC3
      enddo

      call redistaq_VSRM(nbin_aer,dbf_aero,xbf_aero,fixed_rho_aero,dnew,
     $     aerosol)

      
C     14) Update gas & aerosol concentrations in model.
C     -------------------------------------------------

      ZA(ictmHNO3)= gas(ighno3) * 1.d9 * mmHNO3 / cst_RT !HNO3
      ZA(ictmSO2) = gas(igso2)  * 1.d9 * mmSO2  / cst_RT !SO2
      ZA(ictmH2O2)= gas(igh2o2) * 1.d9 * mmH2O2 / cst_RT !H2O2
      ZA(ictmO3)  = gas(igo3)   * 1.d9 * mmO3   / cst_RT !O3 

      DO i = 1,nbin_aer
         ii = ns + i
         ZA(ii+(ENa-1)*nbin_aer) = aerosol(i,nas) ! Na
         ZA(ii+(ESO4-1)*nbin_aer)= aerosol(i,na4) ! SO4
         ZA(ii+(ENH3-1)*nbin_aer)= aerosol(i,naa) ! NH4
         ZA(ii+(ENO3-1)*nbin_aer)= aerosol(i,nan) ! NO3
         ZA(ii+(ECl-1)*nbin_aer) = aerosol(i,nac) ! Cl    
         ZA(ii+(EMD-1)*nbin_aer) = aerosol(i,nar) ! DUST
         ZA(ii+(EBC-1)*nbin_aer) = aerosol(i,nae) ! EC
         ZA(ii+(EPOA-1)*nbin_aer)= aerosol(i,nao) ! POA
         ZA(ii+(EH2O-1)*nbin_aer)= aerosol(i,naw) ! H2O 
      ENDDO

C     Compute in-cloud scavenging
C     -------------------------------
      sulfaf = gas(igso2)      !* 1.d-9 / mmSO2 
      
      do i=1, nbin_aer
         sulfaf = sulfaf + aerosol(i,na4)  * 1.d-9 / mmSO4
      enddo
      sulfaf= sulfaf * mmS

      aqso2 = aqso2 * lwc * 1d3 * mmSO2
      aqh2o2 = aqh2o2 * lwc * 1d3 * mmH2O2
      IF (rain_rate.GT.0.d0) THEN
         ! compute scavenging coefficient, same for all species
         drop_diam = 9.76d-4 * (rain_rate**0.21d0 ) ! m
         scav_coef = 4.17d-7 * collision_eff * rain_rate / drop_diam ! s-1

         ! compute scavenged fraction of droplet
         scav_frac = 1.D0 - dexp(scav_coef * (t0 - t1) ) ! adim

         ! compute scavenged quantity
         DO i = ifirstact,nbin_aer
            ii = ns + i
            DO isp = EMD,EPOA
               qscav(ii+(isp-1)*nbin_aer) = 
     &                       ZA(ii+(isp-1)*nbin_aer) * scav_frac
            ENDDO
         ENDDO
         qscav(ictmSO2) = aqSO2 * scav_frac * 0.7901d0
         qscav(ictmH2O2)= aqH2O2 * scav_frac

                                ! remove scavenged quantity
         DO i = ifirstact,nbin_aer
            ii = ns + i
            DO isp = EMD,EPOA
               ZA(ii+(isp-1)*nbin_aer) = ZA(ii+(isp-1)*nbin_aer) 
     &              - qscav(ii+(isp-1)*nbin_aer)
            ENDDO
         ENDDO
         ZA(ictmSO2) = ZA(ictmSO2) - qscav(ictmSO2)
         ZA(ictmH2O2)= ZA(ictmH2O2) - qscav(ictmH2O2)
      ENDIF

C     Treshold TINYAQ.
C---- 
      DO i=1,NESP
         ZA(i)=dmax1(ZA(i),TINYAQ)
      ENDDO

      return
      end
